/***************************************************************************
                          ktctool.cpp  -  description
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Vairo Raja
    email                : ktctool@fr.ee
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <string.h>

#include <qcstring.h>
#include <qfileinfo.h> 
#include <kaction.h>
#include <kmenubar.h>
#include <kstandarddirs.h>
#include <ktextbrowser.h>
#include <kxmlguiclient.h>

#include "ktctool.h"

// starting application
Ktctool::Ktctool(QWidget *parent, const char *name) : KMainWindow(parent, name)
{

    qdisc_exec=0;
    class_exec=0;
    tcs_exec=1;
    tcd_exec=1;
    tcr_exec=0;
    tcingress_exec=0;
    route_exec=0;
    u32_exec=0;
    tci_exec=0;    
    bounded=0;
    isolated=0;
    dsmarktcindex=0;
    tcdev = 0;
    x = 0;
    filtercount = 0;
    shaperfilterinterfaceChoice = "";  
    shaperfilterrouteChoice = "from";    
    shaperfilteru32Choice = "ip";
    shaperfiltertcChoice = "fall_through";    
    shaperfilteractionChoice = "drop";
    shaperfilterIdChoice = "";    
    shaperdelinterfaceChoice = "";
    pfifobfifoChoice = "pfifo";
    shaperLinks = NULL;
    mainWidget = NULL;
    listall.setAutoDelete(TRUE);
    listinfo.setAutoDelete(TRUE);
    subqdiscs.setAutoDelete(TRUE);
    
    setupConnect();

    startUp();
  
    setupMenuBar();
    setupStatusBar(); 

    //setupKtctoolWidget(); in slotEnable
    //statusbar->message(i18n("Welcome to Ktctool"), 5000);

};


// exit
Ktctool::~Ktctool() {

  delete file;
  delete info;
  delete help;

};


// check user & tc
void Ktctool::startUp() {

  int uid;
  uid = getuid();

  FILE *tc;
  tc = fopen("/sbin/tc", "r");

  // check if user is root
  if(uid>0)
  {
    QMessageBox::critical(0, "Error",
      i18n("You are not running Ktctool as root.\n"));
    exit(0);
  }
  // check if we have tc
  else if (tc==NULL) {
    FILE *tc2;
    FILE *tc3;
    FILE *tc4;
    FILE *tc5;
    tc2 = fopen("/bin/tc", "r");
    tc3 = fopen("/usr/sbin/tc", "r");
    tc4 = fopen("/usr/local/sbin/tc", "r");
    tc5 = fopen("/usr/local/sbin/sbin/tc", "r");
    
    if (tc2==NULL && tc3==NULL && tc4==NULL && tc5==NULL) {
      QMessageBox::critical(0, "Error",
        i18n("Couldn't locate tc.\n"));
      exit(0);
    }
  }
  tc = fopen("/usr/local/kde/bin/ktctool", "r");
  if (tc==NULL) installdir = "./";
  else installdir = "/usr/local/kde/bin/";
  
  tcaction="devices";
  tccommand.clearArguments();
  tccommand << "ip" << "link";
 
  bool successtccommand=tccommand.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Network devices not found!"));
    exit(0);
  }

};


// creating menubar
void Ktctool::setupMenuBar() {

  file = new QPopupMenu();

  file->insertItem(i18n("&QuickHelp"),
          this, SLOT(slotHelp()), Key_F2);
  file->insertSeparator();
  file->insertItem(i18n("&Links"),
          this, SLOT(links()), CTRL+Key_L);
  file->insertItem(i18n("Log &file"),
          this, SLOT(slotLog()), CTRL+Key_F);
  file->insertSeparator();
  file->insertItem(i18n("Refresh QoS &Tree"),
          this, SLOT(startRefresh()), CTRL+Key_T); 
  file->insertItem(i18n("&Reset cells"),
          this, SLOT(slotReset()), CTRL+Key_R);
  file->insertSeparator();          
  file->insertItem(i18n("&Exit"),
          this, SLOT(slotExit()), CTRL+Key_Q);

  info = new QPopupMenu();
  info->insertItem(i18n("&TC"), this, SLOT(slotmantc()));
  info->insertItem(i18n("&CBQ"), this, SLOT(slotmancbq()));
  info->insertItem(i18n("&PFIFO"), this, SLOT(slotmanpfifo()));
  info->insertItem(i18n("p&fifo_fast"), this, SLOT(slotmanpfifof()));
  info->insertItem(i18n("&PRIO"), this, SLOT(slotmanprio()));
  info->insertItem(i18n("&RED"), this, SLOT(slotmanred()));
  info->insertItem(i18n("&SFQ"), this, SLOT(slotmansfq()));
  info->insertItem(i18n("&TBF"), this, SLOT(slotmantbf()));
              
  QPopupMenu *help = helpMenu ();

  menubar = new KMenuBar( this, "menubar" );

  menubar->insertItem(i18n("&File"), file);
  menubar->insertItem(i18n("&Info"), info);  
  menubar->insertItem(i18n("&Help"), help);

};


// create links window
void Ktctool::links() {
 if ( shaperLinks == NULL ) {

  shaperLinks = new QWidget();
  shaperLinks->setCaption(i18n("Links"));

  QBoxLayout *top = new QBoxLayout(shaperLinks, QBoxLayout::LeftToRight, 6);
  QGridLayout * grid = new QGridLayout(6,6);
  top->addLayout(grid,6);

  QLabel *labelbrowser = new QLabel(shaperLinks);
  labelbrowser->setText("                                                                                                                                                                                             ");
  grid->addMultiCellWidget(labelbrowser,0,0,0,6);

  KTextBrowser *browser = new KTextBrowser(shaperLinks);
  browser->setText("<qt> <table border=0><tr><td width=310>Linux Advanced Routing & Traffic Control HOWTO<br> Linux Advanced Routing & Traffic Control list<br> Packet-Shaping-HOWTO<br>\
 QoS Connection Tuning HOWTO<br>\
 Linux - Advanced Networking Overview<br> iproute2+tc notes<br>\
 IP Quality of Service on Linux<br> Stef Coene pages<br> tc - traffic control<br>\
 Linux Traffic Control - Implementation Overview<br> Linux Traffic Control slides<br> Differanted services in Linux<br> IntServ & DiffServ<br>\
 QoS Protocols & Architectures<br> QoS-based Routing Algorithm in Java<br><br>\
 iproute2<br> iptables<br> iptables tutorial<br>\
 knetfilter\
 </td><td width=350> <a href=\"http://lartc.org\">http://www.ds9a.nl/lartc</a><br>  <a href=\"http://mailman.ds9a.nl/mailman/listinfo/lartc/\">http://mailman.ds9a.nl/mailman/listinfo/lartc/</a><br> <a href=\"http://svana.org/kleptog/Packet-Shaping-HOWTO.html\">http://svana.org/kleptog/Packet-Shaping-HOWTO.html</a><br>\
 <a href=\"http://www.prout.be/qos/QoS-connection-tuning-HOWTO.html\">http://www.prout.be/qos/QoS-connection-tuning-HOWTO.html</a><br>\
 <a href=\"http://qos.ittc.ukans.edu/howto/\">http://qos.ittc.ukans.edu/howto/</a><br> <a href=\"http://snafu.freedom.org/linux2.2/iproute-notes.html\">http://snafu.freedom.org/linux2.2/iproute-notes.html</a><br>\
 <a href=\"http://www.davin.ottawa.on.ca/ols/\">http://www.davin.ottawa.on.ca/ols/</a><br> <a href=\"http://docum.org/\">http://docum.org/</a><br> <a href=\"http://www.rns-nis.co.yu/~mps/linux-tc.html\">http://www.rns-nis.co.yu/~mps/linux-tc.html</a><br>\
 <a href=\"ftp://icaftp.epfl.ch/pub/people/almesber/pub/tcio-current.ps.gz\">ftp://icaftp.epfl.ch/pub/people/almesber/pub/tcio-current.ps.gz</a><br> <a href=\"ftp://icaftp.epfl.ch/pub/people/almesber/slides/\">ftp://icaftp.epfl.ch/pub/people/almesber/slides/</a><br> <a href=\"http://diffserv.sourceforge.net/\">http://diffserv.sourceforge.net</a><br> <a href=\"http://www.atm.tut.fi/faster/intserv/\">http://www.atm.tut.fi/faster/intserv/</a><br>\
 <a href=\"http://www.cs.ucsb.edu/~almeroth/classes/F01.201B/papers/qos.pdf\">http://www.cs.ucsb.edu/~almeroth/classes/F01.201B/papers/qos.pdf</a><br> <a href=\"http://www.eden.rutgers.edu/~wfzhang/routing/ver2/qos.html\">http://www.eden.rutgers.edu/~wfzhang/routing/ver2/qos.html</a><br><br>\
 <a href=\"ftp://ftp.inr.ac.ru/ip-routing/\">ftp://ftp.inr.ac.ru/ip-routing/</a><br> <a href=\"http://netfilter.samba.org\">http://netfilter.samba.org</a><br> <a href=\"http://people.unix-fu.org/andreasson/iptables-tutorial/iptables-tutorial.html\">http://people.unix-fu.org/andreasson/iptables-tutorial/iptables-tutorial.html</a><br>\
 <a href=\"http://expansa.sns.it/knetfilter/\">http://expansa.sns.it/knetfilter/</a>\
 <a href=\"\"></a><br></td></tr></table></qt>");
  grid->addMultiCellWidget(browser,0,6,0,6);

  shaperLinks->setGeometry ( 10, 10, 520, 250);
  shaperLinks->show();
  setCentralWidget(shaperLinks);

 } else {
   if ( !shaperLinks->isVisible() )
     shaperLinks->show();
   else
     QMessageBox::critical(0,"Warning", i18n("Already Running\n"));
 }
};

// creating statusbar
void Ktctool::setupStatusBar() {

  statusbar = new KStatusBar( this );

};


// exit
void Ktctool::slotExit() {

  close();
  exit(0);
};


// What's This help
void Ktctool::slotHelp() {

  QWhatsThis::enterWhatsThisMode();

};


// Log file
void Ktctool::slotLog() {

  editlog.clearArguments();
  editlog << "kwrite" << "/root/.ktctool.log";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite /root/.ktctool.log!"));
  }

};


void Ktctool::slotmantc() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc.txt!"));
  }

};


void Ktctool::slotmancbq() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-cbq.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-cbq.txt!"));
  }

};


void Ktctool::slotmanpfifo() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-pbfifo.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-pbfifo.txt!"));
  }

};


void Ktctool::slotmanpfifof() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-pfifo_fast.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-pfifo_fast.txt!"));
  }

};


void Ktctool::slotmanprio() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-prio.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-prio.txt!"));
  }

};


void Ktctool::slotmanred() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-red.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-red.txt!"));
  }

};


void Ktctool::slotmansfq() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-sfq.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-sfq.txt!"));
  }

};


void Ktctool::slotmantbf() {

  editlog.clearArguments();
  editlog << "kwrite" << installdir + "tc-tbf.txt";
  bool successtccommand=editlog.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  if (successtccommand == 0) {
    QMessageBox::critical(0, "Error", i18n("Can't execute kwrite tc-tbf.txt!"));
  }

};

    
// Reset all cell values to default
void Ktctool::slotReset() {
  
qdisc_exec=0;
class_exec=0;
tcs_exec=1;
tcd_exec=1;
tcr_exec=0;
tcingress_exec=0;
route_exec=0;
u32_exec=0;
tci_exec=0;
bounded=0;
isolated=0;
dsmarktcindex=0;
TabWidget2->setCurrentPage(0);
shaperqdiscParent->setText("");
shaperMpu->setText("");
shaperinterface->setCurrentItem(0);
shaperinterfaceActivated(0);
shaperqdiscHandle->setText("1:");
shaperglobalBand->setText("100Mbit");
shaperCell->setText("8");
shaperEwma->setText("");
shaperAvpkt->setText("1000");
dsmarkTcindex->setChecked(FALSE);
dsmarkinterface->setCurrentItem(0);
dsmarkinterfaceActivated(0);
dsmarkqdiscHandle->setText("1:");
dsmarkIndices->setText("");
dsmarkDefaultIndex->setText("");
pfifoqdiscParent->setText("");
pfifointerface->setCurrentItem(0);
pfifointerfaceActivated(0);
pfifobfifo->setCurrentItem(0);
pfifobfifoActivated(0);
pfifoqdiscHandle->setText("1:");
pfifoNumber->setText("");
priointerface->setCurrentItem(0);
priointerfaceActivated(0);
prioqdiscHandle->setText("1:");
priobands->setText("3");
priomap->setText("1 2 1 1  2 2 2 2  0 0 0 0  1 1 1 1");
redqdiscParent->setText("");
redAvpkt->setText("1000");
redBurst->setText("20");
redProbability->setText("0.4");
redglobalBand->setText("3Mbit");
redinterface->setCurrentItem(0);
redinterfaceActivated(0);
redqdiscHandle->setText("1:");
redLimit->setText("60KB");
redMin->setText("15KB");
redMax->setText("45KB");
sfqqdiscParent->setText("");
sfqinterface->setCurrentItem(0);
sfqinterfaceActivated(0);
sfqqdiscHandle->setText("1:");
sfqQuantum->setText("16450");
sfqPerturb->setText("10");
tbfqdiscParent->setText("");
tbfLimit->setText("");
tbfBurst->setText("");
tbfPeakrate->setText("");
tbfLatency->setText("");
tbfMtu->setText("");
tbfinterface->setCurrentItem(0);
tbfinterfaceActivated(0);
tbfqdiscHandle->setText("1:");
tbfRate->setText("200kbit");

shaperclassBounded->setChecked(FALSE);
shaperclassIsolated->setChecked(FALSE);
shaperrateBand->setText("8Mbit");
shaperclassPrio->setText("6");
shaperclassWeight->setText("0.8Mbit");
shaperclassSplit->setText("");
shaperclassDefmap->setText("");
shaperclassMaxburst->setText("20");
shaperclassMinburst->setText("");
shaperinterface->setCurrentItem(0);
shaperinterfaceActivated(0);
shaperclassId->setText("1:10");
shaperparentHandle->setText("");
shaperclassAllot->setText("1514");

shaperfilterinterface->setCurrentItem(0);
shaperfilterinterfaceActivated(0);
shaperfilterprotocol->setText("ip");
shaperfilterprio->setText("1");
shaperfilterclassId->setCurrentItem(0);
shaperfilterhandle->setText("");
shaperfilterroute->setCurrentItem(0);
shaperfilterrouteActivated(0);
shaperfilterroutenr->setText("");
shaperfilteru32->setCurrentItem(0);
shaperfilteru32Activated(0);
shaperfilterU32match->setText("");
shaperfilterpolrate->setText("");
shaperfiltertchash->setText("");
shaperfiltertcmask->setText("");
shaperfiltertcshift->setText("");
shaperfiltertcfall->setCurrentItem(0);
shaperfilterTCIActivated(0);
shaperfilterpolburst->setText("");
shaperfilterpolmtu->setText("");
shaperfilterpolpeakrate->setText("");
shaperfilterpolavrate->setText("");
shaperfilterpolaction->setCurrentItem(0);
shaperfilterpolactionActivated(0);

shaperfilterhandle->setEnabled(FALSE);
shaperfilterroute->setEnabled(FALSE);
shaperfilterroutenr->setEnabled(FALSE);
shaperfilteru32->setEnabled(FALSE);
shaperfilterU32match->setEnabled(FALSE);
shaperfiltertchash->setEnabled(FALSE);
shaperfiltertcmask->setEnabled(FALSE);
shaperfiltertcshift->setEnabled(FALSE);
shaperfiltertcfall->setEnabled(FALSE);
btnfilter->setEnabled(FALSE);
};


// process finished
void Ktctool::slotEnable() {
  Tcobjects *objects;
  QString tcobject,tcobject2, id;

  if (tcaction == "all") {
    if (tcdev < devices.count()) {
      QString dev = devices.at( tcdev );
      tcdev++;
      tccommand.clearArguments();
      if (tcfilter == "ok") {
        tccommand << "tc" << "class" << "show" << "dev" << dev;
        bool successtccommand=tccommand.start(KProcess::NotifyOnExit, KProcess::AllOutput);
        if (successtccommand == 0) {
   //       QMessageBox::critical(0, "Error", "Error: tc class show dev <device>!");
        }
      } else {
        tccommand << "tc" << "filter" << "show" << "dev" << dev;
        bool successtccommand=tccommand.start(KProcess::NotifyOnExit, KProcess::AllOutput);
        if (successtccommand == 0) {
          QMessageBox::critical(0, "Error", "Error: tc filter show dev <device>!");
        }
      }
    } else if (tcfilter == "ok") {
      tcfilter = "";
      tcdev = 0;
      QString dev = devices.at( tcdev );
      tcdev++;
      tccommand.clearArguments();
      tccommand << "tc" << "filter" << "show" << "dev" << dev;
      bool successtccommand=tccommand.start(KProcess::NotifyOnExit, KProcess::AllOutput);
      if (successtccommand == 0) {
        QMessageBox::critical(0, "Error", i18n("Error: tc filter show dev <device>!"));
      }
    } else {  // list all objects in 'Delete'
      QRegExp isclass = QRegExp("[0-9][:][0-9]");
      tcaction = "";
      tcdev = 0;
      shaperdelinterface->clear();
      shaperdelinterface->insertItem(i18n("Select"));
      delobjects.clear();
      delobjects.append("Select");
      for ( objects=listall.first(); objects != 0; objects=listall.next() ) {
        tcobject2.sprintf("%s", objects->type().latin1());
        tcobject.sprintf("dev %s %s %s", objects->dev().latin1(), objects->type().latin1(), objects->id().latin1());
        if (tcobject2 == "class") {
          if (tcobject.contains(isclass)) {  // only classes 1:10, 2:1 etc.
            shaperdelinterface->insertItem(tcobject);
            delobjects.append(tcobject);
          } else {      // only root classes 1:, 2: etc.
          }
        } else {
          shaperdelinterface->insertItem(tcobject);
          delobjects.append(tcobject);
        }
      }
    }
  } else if (tcaction == "classes") {
      QString dev;
      uint classes;
      tcaction = "";
      classes = 0;
      shaperfilterclassId->clear();
      while (classes < listclasses.count()) {
         dev = listclasses.at(classes);
         shaperfilterclassId->insertItem(dev);
         classes++;
      }
         
  } else if (tcaction == "info") {
    tccommand.clearArguments();
    if (tcfilter == "class") {
      tcfilter = "filter";
      tccommand << "tc" << "-d" << "class" << "show" << "dev" << devices.at(tcdev);
      bool successtccommand=tccommand.start(KProcess::NotifyOnExit, KProcess::AllOutput);
      if (successtccommand == 0) {       
 //        QMessageBox::critical(0, "Error", i18n("Error: tc class show!"));
      }
    } else if (tcfilter == "filter") {
      tcfilter = "";
      tccommand << "tc" << "-d" << "filter" << "show" << "dev" << devices.at(tcdev);
      bool successtccommand=tccommand.start(KProcess::NotifyOnExit, KProcess::AllOutput);
      if (successtccommand == 0) {
  //      QMessageBox::critical(0, "Error", i18n("Error: tc filter show!"));
      }
      
    } else {
      if (tcdev < devices.count()-1) { // check next device
        tcdev++;
        startMonitorProcess();
      } else {                     // repaint QoS Tree
        if (x < 1) {   // do when starting      
          x++;         
          tcdev = 0;
          listinfo.clear();
          subqdiscs.clear();
          startMonitorProcess();
        } else {
          setupKtctoolWidget2();
          tcaction = "";
        }
      }
    }
  }
};


// process tc command
void Ktctool::shaperCommand(KProcess *, char *data, int len) {

  if (len<0)
    return;
  static QString remainder;
  QString list;
  QStrList lines;

  char dst[25000];
  memmove(dst,data,len);
  dst[len]=0;

  list=remainder+dst;

  int index=0;
  int newindex=0;

  while (1)
   {
    newindex=list.find('\n',index);
    if(newindex==-1) {
      remainder=list.right(list.length()-index);
      break;
    } else {
      lines.append(list.mid(index,newindex-index));
      index=newindex+1;
    };
  };
  if(lines.count()>0){
        
  if (tcaction == "devices") {
    QString dev;

    for (uint i = 0; i < lines.count(); ++i ) {
       QString data = lines.at( i );
       int k = data.find(" ");
       if (k != 0) {            // not device information
         data.remove(0,k+1);
         k = data.find(" ");
         dev = data.left(k-1);
         if (dev.length() < 7)   // sometimes needed, drop last line
           devices.append( dev );
       }
    }
    devices.sort();
    shaperinterfaceChoice = devices.at(0);
    shapermonitorinterfaceChoice = devices.at(0);
    dsmarkinterfaceChoice = devices.at(0);    
    pfifointerfaceChoice = devices.at(0);
    priointerfaceChoice = devices.at(0);
    redinterfaceChoice = devices.at(0);
    sfqinterfaceChoice = devices.at(0);
    tbfinterfaceChoice = devices.at(0);
    shaperfilterinterfaceChoice = devices.at(0);
  
    setupKtctoolWidget();    // create window
    
  } else if (tcaction == "classes") {
       QString dev;
           
       for (uint i = 0; i < lines.count(); ++i ) {
          QString data = lines.at( i );
          int k = data.find(" ", 7); data.remove(0,k+1);
          k = data.find(" "); dev = data.left(k);
          if (dev.right(1) == ":") {
          } else listclasses.append( dev );
       }
       listclasses.sort();
       if (listclasses.isEmpty()) { }
       else shaperfilterIdChoice = listclasses.at(0);
 
  } else if (tcaction == "all") {  // info which objects to delete
    QString type, dev, id, subqdisc;

    for (uint i = 0; i < lines.count(); ++i ) {
       QString data = lines.at( i );
       int k = data.find(" ");
       type = data.left(k);
       if (type.startsWith("q")) {
         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k); data.remove(0,k+5);
         k = data.find(" ");
         dev = data.left(k);
         listall.append( new Tcobjects(dev, type, id));
       }
       else if (type.startsWith("c")) {
         subqdisc = data;

         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k);
         dev = devices.at(tcdev-1);
         listall.append( new Tcobjects(dev, type, id));

         int j = subqdisc.find("leaf ");
         if (j > 0) {
           subqdisc.remove(0,j+5);
           j = subqdisc.find(" ");
           subqdisc = subqdisc.left(j);  // parent id
           subqdiscs.append(new Tcobjects(dev, subqdisc, id));
         }
       }
       else if (type.startsWith("f")) {
         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k); data.remove(0,k+5);
         dev = devices.at(tcdev-1);
         k = data.find("pref "); data.remove(0,k+5);
         k = data.find(" ");
         id = id + " prio " + data.left(k); data.remove(0,k+1);
         k = data.find("tcindex");
         if ((data.length() < 7) || (k == 0))  //fw, u32, route < 7
           listall.append( new Tcobjects(dev, type, id));
       }
    }
    
   } else if (tcaction == "info") {  // QoS Tree
    QString type, type2, dev, id, subqdisc, info, info2, info3;

    dev = devices.at(tcdev);    
      
       QString data = lines.at( 0 );
       
       info = data; // previous line
       int k = data.find(" ");
       type = data.left(k);
       if (type == "qdisc") {
         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k);
         int p = info.find("qdisc prio ");
         if (p == 0) {
           tcaction = "";
           tcfilter = "";
           QMessageBox::critical(0, "Error", i18n("Sorry, no info about prio at this time!"));
           return;
         }
       }
       else if (type == "class") {
         subqdisc = data;

         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k);
         k = data.find("root");
         if (k > 0) id = "no";  // no need for root class

         int j = subqdisc.find("leaf ");
         if (j > 0) {
           subqdisc.remove(0,j+5);
           j = subqdisc.find(" ");
           subqdisc = subqdisc.left(j);  // parent id
           subqdiscs.append(new Tcobjects(dev, subqdisc, id, info));
         }
       } else if (type == "filter") {
         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k); info3 = data;
         k = info3.find("pref "); info3.remove(0,k+5);
         k = info3.find(" ");
         id = id + " prio " + info3.left(k); info3.remove(0,k+1);
         k = info3.find("32 ");
         if (info3.length() < 7) { info = ""; info2 = "no"; } //fw, u32, route - skip first line
         else if (k > 0) {
           k = info3.find("flowid");
           if (k < 0) { info = ""; info2 = "no"; }  //u32 - skip second line
           else info2 = "yes";
         } else info2 = "yes";
       }

    if (lines.count() == 1) listinfo.append( new Tcobjects(dev, type, id, info));
    uint y = lines.count() - 1;
    
    for (uint i = 1; i < lines.count(); ++i ) {
       data = lines.at( i );

       int k = data.find(" ");
       type2 = data.left(k);
       if (type2 == "qdisc" || type2 == "class" || type2 == "filter") { // write previous line
         if (type2 == "filter" && info2 == "yes")
           listinfo.append( new Tcobjects(dev, type, id, info));
         else if (type2 != "filter")
           listinfo.append( new Tcobjects(dev, type, id, info));         
         info = data;
         type = type2;
       if (type == "qdisc") {
         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k);
         int p = info.find("qdisc prio ");
         if (p == 0) {
           tcaction = "";
           tcfilter = "";
           QMessageBox::critical(0, "Error", i18n("Sorry, no info about prio at this time!"));
           return;
         }
       }
       else if (type == "class") {
         subqdisc = data;

         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k);
         k = data.find("root");
         if (k > 0) id = "no";  // no need for root class

         int j = subqdisc.find("leaf ");
         if (j > 0) {
           subqdisc.remove(0,j+5);
           j = subqdisc.find(" ");
           subqdisc = subqdisc.left(j);  // parent id
           subqdiscs.append(new Tcobjects(dev, subqdisc, id, info));
         }
       } else if (type == "filter") {
         k = data.find(" ",k+1); data.remove(0,k+1);
         k = data.find(" ");
         id = data.left(k); info3 = data;
         k = info3.find("pref "); info3.remove(0,k+5);
         k = info3.find(" ");
         id = id + " prio " + info3.left(k); info3.remove(0,k+1);
         k = info3.find("32 ");
         if (info3.length() < 7) { info = ""; info2 = "no"; } //fw, u32, route - skip first line
         else if (k > 0) {
           k = info3.find("flowid");
           if (k < 0) { info = ""; info2 = "no"; }  //u32 - skip second line
           else info2 = "yes";
         } else info2 = "yes";
       }          
       } else {
         info = info + data;
       }
       if (i == y) listinfo.append( new Tcobjects(dev, type, id, info)); 
    } // end for

   }

  }; // end lines.count()>0

};


// if object exists
void Ktctool::startexistShaper(QString shaperinterface, QString tcid, QString tctype) {
    QRegExp isid = QRegExp("^[1-9][0-9]*[:][0-9]*$");

    if (tcid.contains(isid) || tcid == "ffff:") {
      tcexistid = tcid;
      tcexist.clearArguments();
      tcexist << "tc" << tctype << "show" << "dev" << shaperinterface;
      bool successtccommand=tcexist.start(KProcess::NotifyOnExit, KProcess::AllOutput);
      if (successtccommand == 0) {
        tcexistid = "no";
        QMessageBox::critical(0, "Error", i18n("tc object control failed!"));
      }
    } else {
      tcexistid = "no";
      QMessageBox::critical(0, "Error", i18n("Object handle or id is incorrect, try e.g. 1: or 2:10!"));
    }
};


// if tc object exist
void Ktctool::shaperExist(KProcess *, char *data, int len) {
  if (len<0)
    return;

  static QString remainder;
  QString list;
  QStrList lines;

  char dst[25000];
  memmove(dst,data,len);
  dst[len]=0;

  list=remainder+dst;

  int index=0;
  int newindex=0;

  while (1)
   {
    newindex=list.find('\n',index);
    if(newindex==-1) {
      remainder=list.right(list.length()-index);
      break;
    } else {
      lines.append(list.mid(index,newindex-index));
      index=newindex+1;
    };
  };

  if(lines.count()>0){

    QString type, id;

    for (uint i = 0; i < lines.count(); ++i ) {
       QString data = lines.at( i );
       int k = data.find(" ");
       type = data.left(k);
       k = data.find(" ",k+1); data.remove(0,k+1);
       k = data.find(" ");
       id = data.left(k);
       if (id.compare(tcexistid) == 0)
        {
          tcexistid = "no";
          QMessageBox::critical(0, "Error",
              i18n("The interface already have an object with the same id (handle)! \n"));
        }
    }

  }; // end lines.count()>0

};


// start functions
void Ktctool::shaperExistfinish() {

  if (startfunction == "startshaperRules") startshaperRules();
  else if (startfunction == "dsmarkQdisc") dsmarkQdisc();
  else if (startfunction == "pfifoQdisc") pfifoQdisc();
  else if (startfunction == "prioQdisc") prioQdisc();
  else if (startfunction == "redQdisc") redQdisc();
  else if (startfunction == "sfqQdisc") sfqQdisc();
  else if (startfunction == "tbfQdisc") tbfQdisc();
  else if (startfunction == "ingressQdisc") createIngress();
};


// Change parameters
void Ktctool::setInfo(QString info) {
  QString type, id, id2, id3, device;
  Tcobjects *objects2;

  int k = info.find(" "); device = info.left(k); info.remove(0,k+1);
  uint tcdevice = 0;
  while (tcdevice < devices.count()) {
    QString dev = devices.at( tcdevice );    
    if (device == dev) {
      shapermoninterface->setCurrentItem( tcdevice );
      shapermonitorinterfaceChoice = devices.at( tcdevice );
      break;
    }
    tcdevice++;
  }   
  k = info.find(" "); type = info.left(k); info.remove(0,k+1);
  if (type == "qdisc") {
      TabWidget2->setCurrentPage(1); 
      k = info.find(" "); type = info.left(k); info.remove(0,k+1);
      if (type == "cbq") { // copy cbq info only, change not possible
        shaperqdiscParent->setText("");
        shaperMpu->setText("");
        shaperinterface->setCurrentItem(shapermoninterface->currentItem());
        shaperinterfaceChoice = shapermonitorinterfaceChoice;
                
        k = info.find(" ");  id = info.left(k); shaperqdiscHandle->setText(id);
        for ( objects2=subqdiscs.first(); objects2 != 0; objects2=subqdiscs.next() ) {
           id2.sprintf("%s", objects2->type().latin1());
           if (id2 == id) {   // this qdisc is subqdisc
              id3.sprintf("%s", objects2->id().latin1());
              shaperqdiscParent->setText(id3);
           }
        }
        k = info.find(" rate "); info.remove(0,k+6);
        k = info.find(" "); shaperglobalBand->setText(info.left(k));
        k = info.find(" cell "); info.remove(0,k+6);
        k = info.find(" "); shaperCell->setText(info.left(k));
        k = info.find(" mpu "); if (k > 0) { info.remove(0,k+5);
        k = info.find(" "); shaperMpu->setText(info.left(k)); }
        k = info.find(" ewma "); info.remove(0,k+6);
        k = info.find(" "); shaperEwma->setText(info.left(k));        
        k = info.find(" avpkt "); info.remove(0,k+7);
        k = info.find(" "); shaperAvpkt->setText(info.left(k)); 
        tab_two->setCurrentPage(0);        
     } else if (type == "dsmark") {
        dsmarkTcindex->setChecked(FALSE);
        dsmarkinterface->setCurrentItem(shapermoninterface->currentItem());
        dsmarkinterfaceChoice = shapermonitorinterfaceChoice;
        
        k = info.find(" ");  id = info.left(k); dsmarkqdiscHandle->setText(id);
        k = info.find(" indices "); info.remove(0,k+9);
        k = info.find(" "); dsmarkIndices->setText(info.left(k));
        k = info.find(" default_index "); info.remove(0,k+15);
        k = info.find(" "); dsmarkDefaultIndex->setText(info.left(k));
        k = info.find("set_tc_index"); if (k > 0) { dsmarkTcindex->setChecked(TRUE); }     
        tab_two->setCurrentPage(1);      
     } else if (type == "pfifo" || type == "bfifo") {
        pfifoqdiscParent->setText("");
        pfifointerface->setCurrentItem(shapermoninterface->currentItem());
        pfifointerfaceChoice = shapermonitorinterfaceChoice;
        if (type == "pfifo") {
          pfifobfifo->setCurrentItem(0);
          pfifobfifoActivated(0);
        } else {
          pfifobfifo->setCurrentItem(1);
          pfifobfifoActivated(1);
        }
        
        k = info.find(" ");  id = info.left(k); pfifoqdiscHandle->setText(id);
        for ( objects2=subqdiscs.first(); objects2 != 0; objects2=subqdiscs.next()) {
           id2.sprintf("%s", objects2->type().latin1());
           if (id2 == id) {   // this qdisc is subqdisc
              id3.sprintf("%s", objects2->id().latin1());
              pfifoqdiscParent->setText(id3);
           }
        }
        k = info.find(" limit "); info.remove(0,k+7);
        k = info.find(" "); pfifoNumber->setText(info.left(k));
        tab_two->setCurrentPage(2);        
     } else if (type == "red") {
        redqdiscParent->setText("");
        redAvpkt->setText("");
        redBurst->setText("");
        redProbability->setText("");
        redglobalBand->setText("");
        redinterface->setCurrentItem(shapermoninterface->currentItem());
        redinterfaceChoice = shapermonitorinterfaceChoice;

        k = info.find(" ");  id = info.left(k); redqdiscHandle->setText(id);
        for ( objects2=subqdiscs.first(); objects2 != 0; objects2=subqdiscs.next()) {
           id2.sprintf("%s", objects2->type().latin1());
           if (id2 == id) {   // this qdisc is subqdisc
              id3.sprintf("%s", objects2->id().latin1());
              redqdiscParent->setText(id3);
           }
        }
        k = info.find(" limit "); info.remove(0,k+7);
        k = info.find(" "); redLimit->setText(info.left(k));
        k = info.find(" min "); info.remove(0,k+5);
        k = info.find(" "); redMin->setText(info.left(k));
        k = info.find(" max "); info.remove(0,k+5);
        k = info.find(" "); redMax->setText(info.left(k));
        tab_two->setCurrentPage(4); 
     } else if (type == "sfq") {
        sfqqdiscParent->setText("");
        sfqinterface->setCurrentItem(shapermoninterface->currentItem());
        sfqinterfaceChoice = shapermonitorinterfaceChoice;

        k = info.find(" ");  id = info.left(k); sfqqdiscHandle->setText(id);
        for ( objects2=subqdiscs.first(); objects2 != 0; objects2=subqdiscs.next()) {
           id2.sprintf("%s", objects2->type().latin1());
           if (id2 == id) {   // this qdisc is subqdisc
              id3.sprintf("%s", objects2->id().latin1());
              sfqqdiscParent->setText(id3);
           }
        }
        k = info.find(" quantum "); info.remove(0,k+9);
        k = info.find(" "); sfqQuantum->setText(info.left(k));
        k = info.find(" perturb "); info.remove(0,k+9);
        k = info.find(" "); sfqPerturb->setText(info.left(k));
        tab_two->setCurrentPage(5);
     } else if (type == "tbf") {
        tbfqdiscParent->setText("");
        tbfLimit->setText("");
        tbfBurst->setText("");
        tbfPeakrate->setText("");
        tbfLatency->setText("");
        tbfMtu->setText("");       
        tbfinterface->setCurrentItem(shapermoninterface->currentItem());
        tbfinterfaceChoice = shapermonitorinterfaceChoice;

        k = info.find(" ");  id = info.left(k); tbfqdiscHandle->setText(id);
        for ( objects2=subqdiscs.first(); objects2 != 0; objects2=subqdiscs.next()) {
           id2.sprintf("%s", objects2->type().latin1());
           if (id2 == id) {   // this qdisc is subqdisc
              id3.sprintf("%s", objects2->id().latin1());
              tbfqdiscParent->setText(id3);
           }
        }
        k = info.find(" rate "); info.remove(0,k+6);
        k = info.find(" "); tbfRate->setText(info.left(k));
        k = info.find(" burst "); info.remove(0,k+7);  k = info.find("/");
        tbfBurst->setText(info.left(k));
        tab_two->setCurrentPage(6);
     }                             
  } else if (type == "class") {
      k = info.find(" cbq "); info.remove(0,k+5);
      shaperclassBounded->setChecked(FALSE);
      shaperclassIsolated->setChecked(FALSE);
      shaperclassPrio->setText("");
      shaperclassWeight->setText("");
      shaperclassSplit->setText("");
      shaperclassDefmap->setText("");
      shaperclassMaxburst->setText("");
      shaperclassMinburst->setText("");            
      shaperinterface->setCurrentItem(shapermoninterface->currentItem());
      shaperinterfaceChoice = shapermonitorinterfaceChoice;
            
      k = info.find(" ");  id = info.left(k); shaperclassId->setText(id); info.remove(0,k+8); // remove " parent " too
      k = info.find(" ");  id = info.left(k); shaperparentHandle->setText(id);
      k = info.find(" rate "); info.remove(0,k+6);
      k = info.find(" "); shaperrateBand->setText(info.left(k));
      k = info.find(" cell "); info.remove(0,k+6);
      k = info.find(" "); shaperCell->setText(info.left(k));
      k = info.find("bounded");  if (k > 0) shaperclassBounded->setChecked(TRUE);
      k = info.find("isolated");  if (k > 0) shaperclassIsolated->setChecked(TRUE);
      k = info.find(" prio "); if (k > 0) { info.remove(0,k+6);  k = info.find("/");
          shaperclassPrio->setText(info.left(k)); }
      k = info.find(" weight "); if (k > 0) { info.remove(0,k+8);  k = info.find("bps");
          if (k < 0 || k > 10)  k = info.find(" ");
          shaperclassWeight->setText(info.left(k));  }                       
      k = info.find(" allot "); info.remove(0,k+7);
      k = info.find(" "); shaperclassAllot->setText(info.left(k)); info.remove(0,k+1);      
      k = info.find(" avpkt "); info.remove(0,k+7);
      k = info.find(" "); shaperAvpkt->setText(info.left(k)); info.remove(0,k+1);
      k = info.find(" split "); if (k > 0) { info.remove(0,k+7);  k = info.find(" ");
          shaperclassSplit->setText(info.left(k));  }
      k = info.find(" defmap "); if (k > 0) { info.remove(0,k+8);  k = info.find(" ");
          shaperclassDefmap->setText(info.left(k));  }
      TabWidget2->setCurrentPage(1);          
      tab_two->setCurrentPage(0); 
  } 
};
