/***************************************************************************
                          ktctool_qdisc.cpp  -  description
                             -------------------
    begin                : Fri Feb 1 2002
    copyright            : (C) 2002 by Vairo Raja
    email                : ktctool@fr.ee
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>

#include "ktctool.h"

// dsmark
void Ktctool::dsmarkShaper() {

  QBoxLayout *top = new QBoxLayout(tab_7, QBoxLayout::LeftToRight, 10);
  QGridLayout * grid = new QGridLayout(10,11);
  top->addLayout(grid);

  QLabel *labeldsmarkInterface = new QLabel(tab_7);
  labeldsmarkInterface->setText(i18n("Interface:"));
  grid->addMultiCellWidget(labeldsmarkInterface,0,0,0,0,AlignRight);

  dsmarkinterface = new QComboBox( FALSE, tab_7, "interface");
  dsmarkinterface->insertStrList(devices);
  dsmarkinterface->setFixedHeight(dsmarkinterface->sizeHint().height());
  connect(dsmarkinterface,SIGNAL(activated(int)), SLOT(dsmarkinterfaceActivated(int)));
  grid->addMultiCellWidget(dsmarkinterface,0,0,1,1);

  QLabel *labelend = new QLabel(tab_7);
  labelend->setText("                         ");
  grid->addMultiCellWidget(labelend,0,0,7,10,AlignRight);

  QLabel *labelqdischandle = new QLabel(tab_7);
  labelqdischandle->setText(i18n("qdisc handle:"));
  grid->addMultiCellWidget(labelqdischandle,1,1,0,0,AlignRight);

  dsmarkqdiscHandle = new QLineEdit(tab_7);
  dsmarkqdiscHandle->setFixedHeight(dsmarkqdiscHandle->sizeHint().height());
  dsmarkqdiscHandle->setText("1:");
  grid->addMultiCellWidget(dsmarkqdiscHandle,1,1,1,1);

  QLabel *labelindices = new QLabel(tab_7);
  labelindices->setText("indices:");
  grid->addMultiCellWidget(labelindices,1,1,2,2,AlignRight);

  dsmarkIndices = new QLineEdit(tab_7);
  dsmarkIndices->setFixedHeight(dsmarkIndices->sizeHint().height());
  dsmarkIndices->setText("");
  grid->addMultiCellWidget(dsmarkIndices,1,1,3,3);

  QLabel *labeldefault = new QLabel(tab_7);
  labeldefault->setText(i18n("default_index:"));
  grid->addMultiCellWidget(labeldefault,1,1,4,4,AlignRight);

  dsmarkDefaultIndex = new QLineEdit(tab_7);
  dsmarkDefaultIndex->setFixedHeight(dsmarkDefaultIndex->sizeHint().height());
  dsmarkDefaultIndex->setText("");
  grid->addMultiCellWidget(dsmarkDefaultIndex,1,1,5,5);

  dsmarkTcindex = new QCheckBox("set_tc_index", tab_7);
  dsmarkTcindex->setAutoRepeat( FALSE );
  dsmarkTcindex->setAutoResize( FALSE );
  connect(dsmarkTcindex,SIGNAL(clicked()), this, SLOT(setdsmarkTc()));
  grid->addWidget(dsmarkTcindex,1,6);

  QPushButton *btnShaper = new QPushButton(tab_7);
  btnShaper->setText(i18n("Add qdisc"));
  btnShaper->setFixedHeight(btnShaper->sizeHint().height());
  connect(btnShaper,SIGNAL(clicked()), this, SLOT(startdsmarkQdisc()));
  grid->addWidget(btnShaper,2,2);

  QWhatsThis::add (dsmarkinterface, i18n("Select network interface."));
  QWhatsThis::add (dsmarkqdiscHandle, i18n("This is the handle (major:minor) you want to give it. The minor number must be zero."));
  QWhatsThis::add (btnShaper, i18n("Add qdisc. Root if qdisc parent field is empty."));
  QWhatsThis::add (dsmarkIndices, i18n("Size of table of (mask,value) pairs. Maximum value is 2n, where n>=0. The skb->tc_index value will be initially set by the DSMARK qdisc, retrieving it \
from the DS field in IP header of every received packet. Classifier is invoked. The classifier will be executed and it will return a class ID that will be stored in skb->tc_index variable. \
If no filter matches are found, we consider the default_index option to be the classId to store. If neither set_tc_index nor default_index has been declared results may be unpredictable. \
New_Ds_field = ( Old_DS_field & mask ) | value"));
  QWhatsThis::add (dsmarkDefaultIndex, i18n("The default table entry index if classifier finds no match."));
  QWhatsThis::add (dsmarkTcindex, i18n("Instructs dsmark discipline to retrieve the DS field and store it onto skb->tc_index."));

  tab_7->show();
}


// interface choice
void Ktctool::dsmarkinterfaceActivated(int index) {

  switch(index) {
  default:
    dsmarkinterfaceChoice = devices.at(0);
  case 0:
    dsmarkinterfaceChoice = devices.at(0);
    break;
  case 1:
    dsmarkinterfaceChoice = devices.at(1);
    break;
  case 2:
    dsmarkinterfaceChoice = devices.at(2);
    break;
  case 3:
    dsmarkinterfaceChoice = devices.at(3);
    break;
  case 4:
    dsmarkinterfaceChoice = devices.at(4);
    break;
  case 5:
    dsmarkinterfaceChoice = devices.at(5);
    break;
  case 6:
    dsmarkinterfaceChoice = devices.at(6);
    break;
  case 7:
    dsmarkinterfaceChoice = devices.at(7);
    break;
  case 8:
    dsmarkinterfaceChoice = devices.at(8);
    break;
  };

};


// tcindex checkbox
void Ktctool::setdsmarkTc() {

  if (dsmarktcindex==0)
    {
      dsmarktcindex=1;
    }
  else
    {
      dsmarktcindex=0;
    }

};


// if tc object exist
void Ktctool::startdsmarkQdisc() {
  QString shaperinterface, tcid;

  startfunction = "dsmarkQdisc";
  shaperinterface.sprintf(dsmarkinterfaceChoice);
  tcid.sprintf((const char *)(dsmarkqdiscHandle->text()));
  QString tctype = "qdisc";
  startexistShaper(shaperinterface, tcid, tctype);

};


// create qdisc dsmark
void Ktctool::dsmarkQdisc() {
   QString dsmarkqdischandle, dsmarkindices, dsmarkdefaultindex, start;
   QRegExp isinteger = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itIT]*$");
   QRegExp isbps = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itpsITPS]*$");
   bool successqdisc = false;

   dsmarkqdischandle.sprintf((const char *)(dsmarkqdiscHandle->text()));
   dsmarkindices.sprintf((const char *)(dsmarkIndices->text()));
   dsmarkdefaultindex.sprintf((const char *)(dsmarkDefaultIndex->text()));

   addshaperqdisc.clearArguments();

  if ((dsmarkqdischandle.length()==0)||(dsmarkqdischandle.contains(" ")))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc handle! \n e.g. 1! \n"));
    }
  else
  {
      start = "yes";
      if (tcexistid == "no") start = "no";
      addshaperqdisc << "tc" << "qdisc" << "add";
      addshaperqdisc << "dev" << dsmarkinterfaceChoice;
      addshaperqdisc << "root" << "handle" << dsmarkqdischandle << "dsmark";
      if (dsmarkindices.contains(isinteger)) addshaperqdisc << "indices" << dsmarkindices;
      else {
        QMessageBox::critical(0, "Error",i18n("Indices is not an integer! \n"));
        start = "no";
      }
      if (dsmarkdefaultindex.contains(isinteger)) addshaperqdisc << "default_index" << dsmarkdefaultindex;
      else {
        QMessageBox::critical(0, "Error",i18n("Default_index is not an integer! \n"));
        start = "no";
      }
      if (dsmarktcindex == 1) addshaperqdisc << "set_tc_index";

  }

  if (start == "yes")
    successqdisc=addshaperqdisc.start();

  if (successqdisc) {
    statusbar->message(i18n("New qdisc created"), 2000);
    QString msg;
    msg.sprintf( i18n("qdisc created for interface %s"), dsmarkinterfaceChoice);
    messageLog->insertItem(msg);
    addLog(addshaperqdisc);
    tcdev = 0;
    listinfo.clear();
    subqdiscs.clear();
    startMonitorProcess(); // refresh window
  } else {
    statusbar->message(i18n("tc died"), 2000);
    QMessageBox::critical(0, "Error",
       i18n("qdisc has not been created, please check your parameters! \n"));
  }
};


// pfifo
void Ktctool::pfifoShaper() {

  QBoxLayout *top = new QBoxLayout(tab_2, QBoxLayout::LeftToRight, 10);
  QGridLayout * grid = new QGridLayout(10,11);
  top->addLayout(grid);

  QLabel *labelpfifoInterface = new QLabel(tab_2);
  labelpfifoInterface->setText(i18n("Interface:"));
  grid->addMultiCellWidget(labelpfifoInterface,0,0,0,0,AlignRight);

  pfifointerface = new QComboBox( FALSE, tab_2, "interface");
  pfifointerface->insertStrList(devices);
  pfifointerface->setFixedHeight(pfifointerface->sizeHint().height());
  connect(pfifointerface,SIGNAL(activated(int)), SLOT(pfifointerfaceActivated(int)));
  grid->addMultiCellWidget(pfifointerface,0,0,1,1);

  QLabel *labelend = new QLabel(tab_2);
  labelend->setText("                                                                       ");
  grid->addMultiCellWidget(labelend,0,0,5,10,AlignRight);

  QLabel *labelqdischandle = new QLabel(tab_2);
  labelqdischandle->setText(i18n("qdisc handle:"));
  grid->addMultiCellWidget(labelqdischandle,1,1,0,0,AlignRight);

  pfifoqdiscHandle = new QLineEdit(tab_2);
  pfifoqdiscHandle->setFixedHeight(pfifoqdiscHandle->sizeHint().height());
  pfifoqdiscHandle->setText("1:");
  grid->addMultiCellWidget(pfifoqdiscHandle,1,1,1,1);

  pfifobfifo = new QComboBox( FALSE, tab_2, "bfifo");
  pfifobfifo->insertItem("pfifo");
  pfifobfifo->insertItem("bfifo");
  pfifobfifo->setFixedHeight(pfifobfifo->sizeHint().height());
  connect(pfifobfifo,SIGNAL(activated(int)), SLOT(pfifobfifoActivated(int)));
  grid->addMultiCellWidget(pfifobfifo,1,1,2,2);

  QLabel *labellimit = new QLabel(tab_2);
  labellimit->setText("limit:");
  grid->addMultiCellWidget(labellimit,1,1,3,3,AlignRight);

  pfifoNumber = new QLineEdit(tab_2);
  pfifoNumber->setFixedHeight(pfifoNumber->sizeHint().height());
  pfifoNumber->setText("");
  grid->addMultiCellWidget(pfifoNumber,1,1,4,4);

  QLabel *labelqdiscparent = new QLabel(tab_2);
  labelqdiscparent->setText(i18n("qdisc parent:"));
  grid->addMultiCellWidget(labelqdiscparent,2,2,0,0,AlignRight);

  pfifoqdiscParent = new QLineEdit(tab_2);
  pfifoqdiscParent->setFixedHeight(pfifoqdiscParent->sizeHint().height());
  pfifoqdiscParent->setText("");
  grid->addMultiCellWidget(pfifoqdiscParent,2,2,1,1);

  QPushButton *btnShaper = new QPushButton(tab_2);
  btnShaper->setText(i18n("Add qdisc"));
  btnShaper->setFixedHeight(btnShaper->sizeHint().height());
  connect(btnShaper,SIGNAL(clicked()), this, SLOT(startpfifoQdisc()));
  grid->addWidget(btnShaper,2,2);

  QPushButton *btnChange = new QPushButton(tab_2);
  btnChange->setText( i18n("Change qdisc"));
  btnChange->setFixedHeight(btnChange->sizeHint().height());
  connect(btnChange,SIGNAL(clicked()), this, SLOT(startpfifoQdiscChange()));
  grid->addWidget(btnChange,2,3);

  QWhatsThis::add (pfifointerface, i18n("Select network interface."));
  QWhatsThis::add (pfifoqdiscHandle, i18n("This is the handle (major:minor) you want to give it. The minor number must be zero."));
  QWhatsThis::add (pfifoqdiscParent, i18n("Not required. Parent CLASS ID, e. g. 1:10 (which is specifying the parent of this qdisc). Each qdisc can only be the parent of one other class."));
  QWhatsThis::add (btnShaper, i18n("Add qdisc. Root if qdisc parent field is empty. To set the queue length to 10, execute 'ifconfig eth0 txqueuelen 10'."));
  QWhatsThis::add (btnChange, i18n("Change qdisc parameters. The qdisc must exist and must be the same type."));
  QWhatsThis::add (pfifobfifo, i18n("Select fifo type (packets or bytes)."));
  QWhatsThis::add (pfifoNumber, i18n("Limits the queue size (pkts/bytes)."));

  tab_2->show();
}


// interface choice
void Ktctool::pfifointerfaceActivated(int index) {

  switch(index) {
  default:
    pfifointerfaceChoice = devices.at(0);
  case 0:
    pfifointerfaceChoice = devices.at(0);
    break;
  case 1:
    pfifointerfaceChoice = devices.at(1);
    break;
  case 2:
    pfifointerfaceChoice = devices.at(2);
    break;
  case 3:
    pfifointerfaceChoice = devices.at(3);
    break;
  case 4:
    pfifointerfaceChoice = devices.at(4);
    break;
  case 5:
    pfifointerfaceChoice = devices.at(5);
    break;
  case 6:
    pfifointerfaceChoice = devices.at(6);
    break;
  case 7:
    pfifointerfaceChoice = devices.at(7);
    break;
  case 8:
    pfifointerfaceChoice = devices.at(8);
    break;
  };

};


// fifo choice
void Ktctool::pfifobfifoActivated(int index) {

  switch(index) {
  case 0:
    pfifobfifoChoice = "pfifo";
    break;
  case 1:
    pfifobfifoChoice = "bfifo";
    break;
  };

};


// if tc object exist
void Ktctool::startpfifoQdisc() {
  QString shaperinterface, tcid;

  startfunction = "pfifoQdisc";
  shaperinterface.sprintf(pfifointerfaceChoice);
  tcid.sprintf((const char *)(pfifoqdiscHandle->text()));
  QString tctype = "qdisc";
  startexistShaper(shaperinterface, tcid, tctype);

};


// start changing parameters
void Ktctool::startpfifoQdiscChange() {

  startfunction = "change";
  pfifoQdisc();

};


// create qdisc pfifo
void Ktctool::pfifoQdisc() {
   QString pfifoqdischandle, pfifoqdiscparent, pfifonumber, start;
   QRegExp isinteger = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itIT]*$");
   QRegExp isbps = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itpsITPS]*$");
   bool successqdisc = false;

   pfifoqdischandle.sprintf((const char *)(pfifoqdiscHandle->text()));
   pfifoqdiscparent.sprintf((const char *)(pfifoqdiscParent->text()));
   pfifonumber.sprintf((const char *)(pfifoNumber->text()));

   addshaperqdisc.clearArguments();

  if ((pfifoqdischandle.length()==0)||(pfifoqdischandle.contains(" ")))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc handle! \n e.g. 1! \n"));
    }
  else
  {
      start = "yes";
      if (tcexistid == "no") start = "no";
      addshaperqdisc << "tc" << "qdisc";
      if (startfunction == "change") addshaperqdisc << "change";
      else addshaperqdisc << "add";
      addshaperqdisc << "dev" << pfifointerfaceChoice;
      if (pfifoqdiscparent.length()==0) addshaperqdisc << "root";
      else addshaperqdisc << "parent" << pfifoqdiscparent;
      addshaperqdisc << "handle" << pfifoqdischandle << pfifobfifoChoice;
      if (pfifonumber.contains(isinteger)) addshaperqdisc << "limit" << pfifonumber;
      else {
        QMessageBox::critical(0, "Error",i18n("Limit is not an integer! \n"));
        start = "no";
      }

  }

  if (start == "yes")
    successqdisc=addshaperqdisc.start();

  if (successqdisc) {
    QString msg;
    if (startfunction == "change") {
    statusbar->message(i18n("Qdisc changed"), 2000);
		msg.sprintf( i18n("qdisc changed for interface %s"), pfifointerfaceChoice);
    } else {
    statusbar->message(i18n("New qdisc created"), 2000);
    msg.sprintf( i18n("qdisc created for interface %s"), pfifointerfaceChoice);
    }
    messageLog->insertItem(msg);
    addLog(addshaperqdisc);
    tcdev = 0;
    listinfo.clear();
    subqdiscs.clear();    
    startMonitorProcess(); // refresh window
  } else {
    statusbar->message(i18n("tc died"), 2000);
    QMessageBox::critical(0, "Error",
       i18n("qdisc has not been created, please check your parameters! \n"));
  }
};


// prio
void Ktctool::prioShaper() {

   QBoxLayout *top = new QBoxLayout(tab_3, QBoxLayout::LeftToRight, 10);
  QGridLayout * grid = new QGridLayout(10,11);
  top->addLayout(grid);

  QLabel *labelprioInterface = new QLabel(tab_3);
  labelprioInterface->setText(i18n("Interface:"));
  grid->addMultiCellWidget(labelprioInterface,0,0,0,0,AlignRight);

  priointerface = new QComboBox( FALSE, tab_3, "interface");
  priointerface->insertStrList(devices);
  priointerface->setFixedHeight(priointerface->sizeHint().height());
  connect(priointerface,SIGNAL(activated(int)), SLOT(priointerfaceActivated(int)));
  grid->addMultiCellWidget(priointerface,0,0,1,1);

  QLabel *labelend = new QLabel(tab_3);
  labelend->setText("                                                   ");
  grid->addMultiCellWidget(labelend,0,0,5,6,AlignRight);

  QLabel *labelend2 = new QLabel(tab_3);
  labelend2->setText("           ");
  grid->addMultiCellWidget(labelend2,0,0,7,10,AlignRight);

  QLabel *labelqdischandle = new QLabel(tab_3);
  labelqdischandle->setText(i18n("qdisc handle:"));
  grid->addMultiCellWidget(labelqdischandle,1,1,0,0,AlignRight);

  prioqdiscHandle = new QLineEdit(tab_3);
  prioqdiscHandle->setFixedHeight(prioqdiscHandle->sizeHint().height());
  prioqdiscHandle->setText("1:");
  grid->addMultiCellWidget(prioqdiscHandle,1,1,1,1);

  QLabel *labelbands = new QLabel(tab_3);
  labelbands->setText("    bands:");
  grid->addMultiCellWidget(labelbands,1,1,2,2,AlignRight);

  priobands = new QLineEdit(tab_3);
  priobands->setFixedHeight(priobands->sizeHint().height());
  priobands->setText("3");
  grid->addMultiCellWidget(priobands,1,1,3,3);

  QLabel *labelpriomap = new QLabel(tab_3);
  labelpriomap->setText("   priomap:");
  grid->addMultiCellWidget(labelpriomap,1,1,4,4,AlignRight);

  priomap = new QLineEdit(tab_3);
  priomap->setFixedHeight(priomap->sizeHint().height());
  priomap->setText("1 2 1 1  2 2 2 2  0 0 0 0  1 1 1 1");
  grid->addMultiCellWidget(priomap,1,1,5,6);

  QPushButton *btnShaper = new QPushButton(tab_3);
  btnShaper->setText( i18n("Add qdisc"));
  btnShaper->setFixedHeight(btnShaper->sizeHint().height());
  connect(btnShaper,SIGNAL(clicked()), this, SLOT(startprioQdisc()));
  grid->addWidget(btnShaper,2,2);

  QPushButton *btnChange = new QPushButton(tab_3);
  btnChange->setText( i18n("Change qdisc"));
  btnChange->setFixedHeight(btnChange->sizeHint().height());
  connect(btnChange,SIGNAL(clicked()), this, SLOT(startprioQdiscChange()));
  grid->addWidget(btnChange,2,3);

  QWhatsThis::add (priointerface, i18n("Select network interface."));
  QWhatsThis::add (prioqdiscHandle, i18n("This is the handle (major:minor) you want to give it. The minor number must be zero."));
  QWhatsThis::add (btnShaper, i18n("Add qdisc. After creating prio 1: you can create qdiscs using parent 1:1, 1:2, 1:3."));
  QWhatsThis::add (btnChange, i18n("Change qdisc parameters. The qdisc must exist and must be the same type."));
  QWhatsThis::add (priobands, i18n("By default, three classes (bands) are created. If you change this number, you must also change priomap. After creating prio you can create qdiscs using parent 1:1, 1:2, 1:3."));
  QWhatsThis::add (priomap, i18n("<qt>Default priomap: '1 2 2 2 &nbsp;1 2 0 0 &nbsp;1 1 1 1 &nbsp;1 1 1 1'.<br/><pre>\
<b>TOS     Bits  Means                    Linux Priority    Band</b><br/>\
------------------------------------------------------------<br/>\
0x0     0     Normal Service           0 Best Effort     1  <br/>\
0x2     1     Minimize Monetary Cost   1 Filler          2  <br/>\
0x4     2     Maximize Reliability     0 Best Effort     1  <br/>\
0x6     3     mmc+mr                   0 Best Effort     1  <br/>\
0x8     4     Maximize Throughput      2 Bulk            2  <br/>\
0xa     5     mmc+mt                   2 Bulk            2  <br/>\
0xc     6     mr+mt                    2 Bulk            2  <br/>\
0xe     7     mmc+mr+mt                2 Bulk            2  <br/>\
0x10    8     Minimize Delay           6 Interactive     0  <br/>\
0x12    9     mmc+md                   6 Interactive     0  <br/>\
0x14    10    mr+md                    6 Interactive     0  <br/>\
0x16    11    mmc+mr+md                6 Interactive     0  <br/>\
0x18    12    mt+md                    4 Int. Bulk       1  <br/>\
0x1a    13    mmc+mt+md                4 Int. Bulk       1  <br/>\
0x1c    14    mr+mt+md                 4 Int. Bulk       1  <br/>\
0x1e    15    mmc+mr+mt+md             4 Int. Bulk       1  <br/>\
</pre> The second column contains the value of the relevant four TOS bits. The fourth column lists the way the Linux kernel interprets the TOS bits, by showing to which Priority they are mapped. \
The last column shows the result of the default priomap. This means that priority 4, for example, gets mapped to band number 1.<br/> By default, <b>three</b> classes (bands) are created. <b>If you change \
this number smaller, you must also change priomap, e.g. if 2 classes then priomap only consists of 0 & 1.</b><br/> The PRIO qdisc doesn't actually shape, it only subdivides traffic. These classes by default contain pure FIFO \
qdiscs with no internal structure, but you can replace these by any qdisc you have available. Whenever a packet needs to be dequeued, class :1 is tried first. </qt>"));

  tab_3->show();
};


// interface choice
void Ktctool::priointerfaceActivated(int index) {

  switch(index) {
  default:
    priointerfaceChoice = devices.at(0);
  case 0:
    priointerfaceChoice = devices.at(0);
    break;
  case 1:
    priointerfaceChoice = devices.at(1);
    break;
  case 2:
    priointerfaceChoice = devices.at(2);
    break;
  case 3:
    priointerfaceChoice = devices.at(3);
    break;
  case 4:
    priointerfaceChoice = devices.at(4);
    break;
  case 5:
    priointerfaceChoice = devices.at(5);
    break;
  case 6:
    priointerfaceChoice = devices.at(6);
    break;
  case 7:
    priointerfaceChoice = devices.at(7);
    break;
  case 8:
    priointerfaceChoice = devices.at(8);
    break;
  };

};


// if tc object exist
void Ktctool::startprioQdisc() {
  QString shaperinterface, tcid;

  startfunction = "prioQdisc";
  shaperinterface.sprintf(priointerfaceChoice);
  tcid.sprintf((const char *)(prioqdiscHandle->text()));
  QString tctype = "qdisc";
  startexistShaper(shaperinterface, tcid, tctype);

};


// start changing parameters
void Ktctool::startprioQdiscChange() {

  startfunction = "change";
  prioQdisc();

};


// create qdisc prio
void Ktctool::prioQdisc() {
   QString prioqdischandle, priomapq, priobandsq, start;
   QRegExp isinteger = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itIT]*$");
   QRegExp isbps = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itpsITPS]*$");
   bool successqdisc = false;

   prioqdischandle.sprintf((const char *)(prioqdiscHandle->text()));
   priobandsq.sprintf((const char *)(priobands->text()));
   priomapq.sprintf((const char *)(priomap->text()));

   addshaperqdisc.clearArguments();

  if ((prioqdischandle.length()==0)||(prioqdischandle.contains(" ")))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc handle! \n e.g. 1! \n"));
    }
  else
  {
      start = "yes";
      if (tcexistid == "no") start = "no";
      addshaperqdisc << "tc" << "qdisc";
      if (startfunction == "change") addshaperqdisc << "change";
      else addshaperqdisc << "add";
      addshaperqdisc << "dev" << priointerfaceChoice;
      addshaperqdisc << "root";
      addshaperqdisc << "handle" << prioqdischandle << "prio";
      if (priobandsq != "3")
        if (priobandsq.contains(isinteger)) addshaperqdisc << "bands" << priobandsq;
        else {
          QMessageBox::critical(0, "Error", i18n("Bands is not an integer! \n"));
          start = "no";
        }
      if (priomapq != "") {
        int k; k = priomapq.find(" ");
        addshaperqdisc << "priomap";
        while (k > -1) {
          if (k == 0) priomapq.remove(0,k+1);
          else {
            if (priomapq.left(k) >= priobandsq) {
              QMessageBox::critical(0, "Error", i18n("Priomap is out of range!"));
              start = "no";
              break;
            }
            addshaperqdisc << priomapq.left(k); priomapq.remove(0,k+1);
          }
          k = priomapq.find(" ");
        }
        addshaperqdisc << priomapq;
      }
  }

  if (start == "yes")
    successqdisc=addshaperqdisc.start();

  if (successqdisc) {
    QString msg;
    if (startfunction == "change") {
    statusbar->message(i18n("Qdisc changed"), 2000);
		msg.sprintf( i18n("qdisc changed for interface %s"), priointerfaceChoice);
    } else {
    statusbar->message(i18n("New qdisc created"), 2000);
    msg.sprintf( i18n("qdisc created for interface %s"), priointerfaceChoice);
    }
    messageLog->insertItem(msg);
    addLog(addshaperqdisc);
    tcdev = 0;
    listinfo.clear();
    subqdiscs.clear();    
    startMonitorProcess(); // refresh window
    
  } else {
    statusbar->message(i18n("tc died"), 2000);
    QMessageBox::critical(0, "Error",
       i18n("qdisc has not been created, please check your parameters! \n"));
  }
};


// red
void Ktctool::redShaper() {

  QBoxLayout *top = new QBoxLayout(tab_4, QBoxLayout::LeftToRight, 10);
  QGridLayout * grid = new QGridLayout(10,11);
  top->addLayout(grid);

  QLabel *labelredInterface = new QLabel(tab_4);
  labelredInterface->setText(i18n("Interface:"));
  grid->addMultiCellWidget(labelredInterface,0,0,0,0,AlignRight);

  redinterface = new QComboBox( FALSE, tab_4, "interface");
  redinterface->insertStrList(devices);
  redinterface->setFixedHeight(redinterface->sizeHint().height());
  connect(redinterface,SIGNAL(activated(int)), SLOT(redinterfaceActivated(int)));
  grid->addMultiCellWidget(redinterface,0,0,1,1);

  QLabel *labelend = new QLabel(tab_4);
  labelend->setText("           ");
  grid->addMultiCellWidget(labelend,0,0,8,10,AlignRight);

  QLabel *labelredglobalBand = new QLabel(tab_4);
  labelredglobalBand->setText("bandwidth:");
  grid->addMultiCellWidget(labelredglobalBand,0,0,2,2,AlignRight);

  redglobalBand = new QLineEdit(tab_4);
  redglobalBand->setFixedHeight(redglobalBand->sizeHint().height());
  redglobalBand->setText("3Mbit");
  grid->addMultiCellWidget(redglobalBand,0,0,3,3);

  QLabel *labelmin = new QLabel(tab_4);
  labelmin ->setText("min:");
  grid->addMultiCellWidget(labelmin,0,0,4,4,AlignRight);

  redMin = new QLineEdit(tab_4);
  redMin->setFixedHeight(redMin->sizeHint().height());
  redMin->setText("15KB");
  grid->addMultiCellWidget(redMin,0,0,5,5);

  QLabel *labelavpkt = new QLabel(tab_4);
  labelavpkt ->setText("avpkt:");
  grid->addMultiCellWidget(labelavpkt,0,0,6,6,AlignRight);

  redAvpkt = new QLineEdit(tab_4);
  redAvpkt->setFixedHeight(redAvpkt->sizeHint().height());
  redAvpkt->setText("1000");
  grid->addMultiCellWidget(redAvpkt,0,0,7,7);

  QLabel *labelqdischandle = new QLabel(tab_4);
  labelqdischandle->setText(i18n("qdisc handle:"));
  grid->addMultiCellWidget(labelqdischandle,1,1,0,0,AlignRight);

  redqdiscHandle = new QLineEdit(tab_4);
  redqdiscHandle->setFixedHeight(redqdiscHandle->sizeHint().height());
  redqdiscHandle->setText("1:");
  grid->addMultiCellWidget(redqdiscHandle,1,1,1,1);

  QLabel *labellimit = new QLabel(tab_4);
  labellimit->setText("limit:");
  grid->addMultiCellWidget(labellimit,1,1,2,2,AlignRight);

  redLimit = new QLineEdit(tab_4);
  redLimit->setFixedHeight(redLimit->sizeHint().height());
  redLimit->setText("60KB");
  grid->addMultiCellWidget(redLimit,1,1,3,3);

  QLabel *labelmax = new QLabel(tab_4);
  labelmax ->setText("max:");
  grid->addMultiCellWidget(labelmax,1,1,4,4,AlignRight);

  redMax = new QLineEdit(tab_4);
  redMax->setFixedHeight(redMax->sizeHint().height());
  redMax->setText("45KB");
  grid->addMultiCellWidget(redMax,1,1,5,5);

  QLabel *labelburst = new QLabel(tab_4);
  labelburst ->setText("burst:");
  grid->addMultiCellWidget(labelburst,1,1,6,6,AlignRight);

  redBurst = new QLineEdit(tab_4);
  redBurst->setFixedHeight(redBurst->sizeHint().height());
  redBurst->setText("20");
  grid->addMultiCellWidget(redBurst,1,1,7,7);

  QLabel *labelqdiscparent = new QLabel(tab_4);
  labelqdiscparent->setText(i18n("qdisc parent:"));
  grid->addMultiCellWidget(labelqdiscparent,2,2,0,0,AlignRight);

  redqdiscParent = new QLineEdit(tab_4);
  redqdiscParent->setFixedHeight(redqdiscParent->sizeHint().height());
  redqdiscParent->setText("");
  grid->addMultiCellWidget(redqdiscParent,2,2,1,1);

  QLabel *labelprobability = new QLabel(tab_4);
  labelprobability->setText("probability:");
  grid->addMultiCellWidget(labelprobability,2,2,2,2,AlignRight);

  redProbability = new QLineEdit(tab_4);
  redProbability->setFixedHeight(redProbability->sizeHint().height());
  redProbability->setText("0.4");
  grid->addMultiCellWidget(redProbability,2,2,3,3);

  QPushButton *btnShaper = new QPushButton(tab_4);
  btnShaper->setText( i18n("Add qdisc"));
  btnShaper->setFixedHeight(btnShaper->sizeHint().height());
  connect(btnShaper,SIGNAL(clicked()), this, SLOT(startredQdisc()));
  grid->addWidget(btnShaper,3,2);

  QPushButton *btnChange = new QPushButton(tab_4);
  btnChange->setText( i18n("Change qdisc"));
  btnChange->setFixedHeight(btnChange->sizeHint().height());
  connect(btnChange,SIGNAL(clicked()), this, SLOT(startredQdiscChange()));
  grid->addWidget(btnChange,3,3);

  QWhatsThis::add (redinterface, i18n("Select network interface."));
  QWhatsThis::add (redqdiscHandle, i18n("This is the handle (major:minor) you want to give it. The minor number must be zero."));
  QWhatsThis::add (redqdiscParent, i18n("Not required. Parent CLASS ID, e. g. 1:10 (which is specifying the parent of this qdisc). Each qdisc can only be the parent of one other class."));
  QWhatsThis::add (btnShaper, i18n("Add qdisc. Root if qdisc parent field is empty. \n Example: tc qdisc add dev eth0 parent 1:10 red limit 60KB min 15KB max 45KB burst 20 avpkt 1000 bandwidth 3Mbit probability 0.4"));
  QWhatsThis::add (btnChange, i18n("Change qdisc parameters. The qdisc must exist and must be the same type."));
  QWhatsThis::add (redglobalBand, i18n("The physical bandwidth of your interface."));
  QWhatsThis::add (redMin, i18n("The min avg threshold (bytes) < max/2"));
  QWhatsThis::add (redMax, i18n("The max avg threshold (bytes) > 2*min & < limit"));
  QWhatsThis::add (redAvpkt, i18n("Average packet size. Default 1000."));
  QWhatsThis::add (redLimit, i18n("The physical queue size (bytes), recommended > (max + burst)."));
  QWhatsThis::add (redBurst, i18n("Used to compute time constant (packets). \n burst + 1 - min/avpkt < (1-(0.5)^burst)/0.5 \n\
 Example: tc qdisc add dev eth0 parent 1:10 red limit 60KB min 15KB max 45KB burst 20 avpkt 1000 bandwidth 3Mbit probability 0.4"));
  QWhatsThis::add (redProbability, i18n("Not required. Probability (float, e.g. 0.02), random drop constant. \n Plog = log(prob/(max - min))"));

  tab_4->show();
}


// interface choice
void Ktctool::redinterfaceActivated(int index) {

  switch(index) {
  default:
    redinterfaceChoice = devices.at(0);
  case 0:
    redinterfaceChoice = devices.at(0);
    break;
  case 1:
    redinterfaceChoice = devices.at(1);
    break;
  case 2:
    redinterfaceChoice = devices.at(2);
    break;
  case 3:
    redinterfaceChoice = devices.at(3);
    break;
  case 4:
    redinterfaceChoice = devices.at(4);
    break;
  case 5:
    redinterfaceChoice = devices.at(5);
    break;
  case 6:
    redinterfaceChoice = devices.at(6);
    break;
  case 7:
    redinterfaceChoice = devices.at(7);
    break;
  case 8:
    redinterfaceChoice = devices.at(8);
    break;
  };

};


// if tc object exist
void Ktctool::startredQdisc() {
  QString shaperinterface, tcid;

  startfunction = "redQdisc";
  shaperinterface.sprintf(redinterfaceChoice);
  tcid.sprintf((const char *)(redqdiscHandle->text()));
  QString tctype = "qdisc";
  startexistShaper(shaperinterface, tcid, tctype);

};


// start changing parameters
void Ktctool::startredQdiscChange() {

  startfunction = "change";
  redQdisc();

};


// create qdisc red
void Ktctool::redQdisc() {
   QString redqdischandle, redqdiscparent, redglobalband, redmin, redmax, redavpkt, redlimit, redburst, redprobability, start;
   QRegExp isinteger = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itIT]*$");
   QRegExp isbps = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itpsITPS]*$");
   bool successqdisc = false;

   redqdischandle.sprintf((const char *)(redqdiscHandle->text()));
   redqdiscparent.sprintf((const char *)(redqdiscParent->text()));
   redglobalband.sprintf((const char *)(redglobalBand->text()));
   redmin.sprintf((const char *)(redMin->text()));
   redmax.sprintf((const char *)(redMax->text()));
   redavpkt.sprintf((const char *)(redAvpkt->text()));
   redlimit.sprintf((const char *)(redLimit->text()));
   redburst.sprintf((const char *)(redBurst->text()));
   redprobability.sprintf((const char *)(redProbability->text()));

   addshaperqdisc.clearArguments();

  if ((redqdischandle.length()==0)||(redqdischandle.contains(" ")))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc handle! \n e.g. 1! \n"));
    }
  else
  {
      start = "yes";
      if (tcexistid == "no") start = "no";

      addshaperqdisc << "tc" << "qdisc";
      if (startfunction == "change") addshaperqdisc << "change";
      else addshaperqdisc << "add";
      addshaperqdisc << "dev" << redinterfaceChoice;
      if (redqdiscparent.length()==0) addshaperqdisc << "root";
      else addshaperqdisc << "parent" << redqdiscparent;
      addshaperqdisc << "handle" << redqdischandle << "red";
      if ((redglobalband.length()==0)||(redmin.length()==0)||(redmax.length()==0)||(redavpkt.length()==0)||(redlimit.length()==0)||(redburst.length()==0)) {
          QMessageBox::critical(0, "Error", i18n("All parameters required, except probability. \n"));
          start = "no";
      } else {
        if (redglobalband.contains(isbps)) addshaperqdisc << "bandwidth" << redglobalband;
        else {
          QMessageBox::critical(0, "Error", i18n("Wrong bandwidth! \n"));
          start = "no";
        }
        if (redmin.contains(isinteger)) addshaperqdisc << "min" << redmin;
        else {
          QMessageBox::critical(0, "Error", i18n("Min is not an integer! \n"));
          start = "no";
        }
        if (redmax.contains(isinteger)) addshaperqdisc << "max" << redmax;
        else {
          QMessageBox::critical(0, "Error", i18n("Max is not an integer! \n"));
          start = "no";
        }
        if (redavpkt.contains(isinteger)) addshaperqdisc << "avpkt" << redavpkt;
        else {
          QMessageBox::critical(0, "Error", i18n("Avpkt is not an integer! \n"));
          start = "no";
        }
        if (redlimit.contains(isinteger)) addshaperqdisc << "limit" << redlimit;
        else {
          QMessageBox::critical(0, "Error", i18n("Limit is not an integer! \n"));
          start = "no";
        }
        if (redburst.contains(isinteger)) addshaperqdisc << "burst" << redburst;
        else {
          QMessageBox::critical(0, "Error", i18n("Burst is not an integer! \n"));
          start = "no";
        }
      }
  }

  if (start == "yes")
    successqdisc=addshaperqdisc.start();
  if (successqdisc) {
    QString msg;
    if (startfunction == "change") {
    statusbar->message(i18n("Qdisc changed"), 2000);
		msg.sprintf( i18n("qdisc changed for interface %s"), redinterfaceChoice);
    } else {
    statusbar->message(i18n("New qdisc created"), 2000);
    msg.sprintf( i18n("qdisc created for interface %s"), redinterfaceChoice);
    }
    messageLog->insertItem(msg);
    addLog(addshaperqdisc);
    tcdev = 0;
    listinfo.clear();
    subqdiscs.clear();    
    startMonitorProcess(); // refresh window
    
  } else {
    statusbar->message(i18n("tc died"), 2000);
    QMessageBox::critical(0, "Error",
       i18n("qdisc has not been created, please check your parameters! \n"));
  }
};


// sfq
void Ktctool::sfqShaper() {

  QBoxLayout *top = new QBoxLayout(tab_5, QBoxLayout::LeftToRight, 10);
  QGridLayout * grid = new QGridLayout(10,11);
  top->addLayout(grid);

  QLabel *labelsfqInterface = new QLabel(tab_5);
  labelsfqInterface->setText(i18n("Interface:"));
  grid->addMultiCellWidget(labelsfqInterface,0,0,0,0,AlignRight);

  sfqinterface = new QComboBox( FALSE, tab_5, "interface");
  sfqinterface->insertStrList(devices);
  sfqinterface->setFixedHeight(sfqinterface->sizeHint().height());
  connect(sfqinterface,SIGNAL(activated(int)), SLOT(sfqinterfaceActivated(int)));
  grid->addMultiCellWidget(sfqinterface,0,0,1,1);

  QLabel *labelend = new QLabel(tab_5);
  labelend->setText("                                               ");
  grid->addMultiCellWidget(labelend,0,0,6,10,AlignRight);

  QLabel *labelqdischandle = new QLabel(tab_5);
  labelqdischandle->setText(i18n("qdisc handle:"));
  grid->addMultiCellWidget(labelqdischandle,1,1,0,0,AlignRight);

  sfqqdiscHandle = new QLineEdit(tab_5);
  sfqqdiscHandle->setFixedHeight(sfqqdiscHandle->sizeHint().height());
  sfqqdiscHandle->setText("1:");
  grid->addMultiCellWidget(sfqqdiscHandle,1,1,1,1);

  QLabel *labelperturb = new QLabel(tab_5);
  labelperturb->setText("perturb:");
  grid->addMultiCellWidget(labelperturb,1,1,2,2,AlignRight);

  sfqPerturb = new QLineEdit(tab_5);
  sfqPerturb->setFixedHeight(sfqPerturb->sizeHint().height());
  sfqPerturb->setText("10");
  grid->addMultiCellWidget(sfqPerturb,1,1,3,3);

  QLabel *labelquantum = new QLabel(tab_5);
  labelquantum->setText("quantum:");
  grid->addMultiCellWidget(labelquantum,1,1,4,4,AlignRight);

  sfqQuantum = new QLineEdit(tab_5);
  sfqQuantum->setFixedHeight(sfqQuantum->sizeHint().height());
  sfqQuantum->setText("16450");
  grid->addMultiCellWidget(sfqQuantum,1,1,5,5);

  QLabel *labelqdiscparent = new QLabel(tab_5);
  labelqdiscparent->setText(i18n("qdisc parent:"));
  grid->addMultiCellWidget(labelqdiscparent,2,2,0,0,AlignRight);

  sfqqdiscParent = new QLineEdit(tab_5);
  sfqqdiscParent->setFixedHeight(sfqqdiscParent->sizeHint().height());
  sfqqdiscParent->setText("");
  grid->addMultiCellWidget(sfqqdiscParent,2,2,1,1);

  QPushButton *btnShaper = new QPushButton(tab_5);
  btnShaper->setText( i18n("Add qdisc"));
  btnShaper->setFixedHeight(btnShaper->sizeHint().height());
  connect(btnShaper,SIGNAL(clicked()), this, SLOT(startsfqQdisc()));
  grid->addWidget(btnShaper,2,2);

  QWhatsThis::add (sfqinterface, i18n("Select network interface."));
  QWhatsThis::add (sfqqdiscHandle, i18n("This is the handle (major:minor) you want to give it. The minor number must be zero."));
  QWhatsThis::add (sfqqdiscParent, i18n("Not required. Parent CLASS ID, e. g. 1:10 (which is specifying the parent of this qdisc). Each qdisc can only be the parent of one other class."));
  QWhatsThis::add (btnShaper, i18n("Add qdisc. Root if qdisc parent field is empty.The key word in SFQ is conversation (or flow), which mostly corresponds to a TCP session or a UDP stream. Traffic is divided into a pretty large number of FIFO queues,\
 one for each conversation. Traffic is then sent in a round robin fashion, giving each session the chance to send data in turn. SFQ is only useful in case your actual outgoing interface is really full!"));
  QWhatsThis::add (sfqPerturb, i18n("Reconfigure hashing once this many seconds. If unset, hash will never be reconfigured. Not recommended. 10 seconds is probably a good value."));
  QWhatsThis::add (sfqQuantum, i18n("Amount of bytes a stream is allowed to dequeue before the next queue gets a turn. Defaults to 1 maximum sized packet (MTU-sized). Do not set below the MTU!"));

  tab_5->show();
}


// interface choice
void Ktctool::sfqinterfaceActivated(int index) {

  switch(index) {
  default:
    sfqinterfaceChoice = devices.at(0);
  case 0:
    sfqinterfaceChoice = devices.at(0);
    break;
  case 1:
    sfqinterfaceChoice = devices.at(1);
    break;
  case 2:
    sfqinterfaceChoice = devices.at(2);
    break;
  case 3:
    sfqinterfaceChoice = devices.at(3);
    break;
  case 4:
    sfqinterfaceChoice = devices.at(4);
    break;
  case 5:
    sfqinterfaceChoice = devices.at(5);
    break;
  case 6:
    sfqinterfaceChoice = devices.at(6);
    break;
  case 7:
    sfqinterfaceChoice = devices.at(7);
    break;
  case 8:
    sfqinterfaceChoice = devices.at(8);
    break;
  };

};


// if tc object exist
void Ktctool::startsfqQdisc() {
  QString shaperinterface, tcid;

  startfunction = "sfqQdisc";
  shaperinterface.sprintf(sfqinterfaceChoice);
  tcid.sprintf((const char *)(sfqqdiscHandle->text()));
  QString tctype = "qdisc";
  startexistShaper(shaperinterface, tcid, tctype);

};


// create qdisc sfq
void Ktctool::sfqQdisc() {
   QString sfqqdischandle, sfqqdiscparent, sfqperturb, sfqquantum, start;
   QRegExp isinteger = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itIT]*$");
   QRegExp isbps = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itpsITPS]*$");
   bool successqdisc = false;

   sfqqdischandle.sprintf((const char *)(sfqqdiscHandle->text()));
   sfqqdiscparent.sprintf((const char *)(sfqqdiscParent->text()));
   sfqperturb.sprintf((const char *)(sfqPerturb->text()));
   sfqquantum.sprintf((const char *)(sfqQuantum->text()));

   addshaperqdisc.clearArguments();

  if ((sfqqdischandle.length()==0)||(sfqqdischandle.contains(" ")))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc handle! \n e.g. 1! \n"));
    }
  else
  {
      start = "yes";
      if (tcexistid == "no") start = "no";

      addshaperqdisc << "tc" << "qdisc" << "add" << "dev" << sfqinterfaceChoice;
      if (sfqqdiscparent.length()==0) addshaperqdisc << "root";
      else addshaperqdisc << "parent" << sfqqdiscparent;
      addshaperqdisc << "handle" << sfqqdischandle << "sfq";
      if (sfqperturb.length()!=0)
        addshaperqdisc << "perturb" << sfqperturb;
      else {
          QMessageBox::critical(0, "Error", i18n("Perturb is not an integer! \n"));
          start = "no";
        }
      if (sfqquantum.length()!=0)
        if (sfqquantum.contains(isinteger)) addshaperqdisc << "quantum" << sfqquantum;
        else {
          QMessageBox::critical(0, "Error", i18n("Quantum is not an integer! \n"));
          start = "no";
        }
  }

  if (start == "yes")
    successqdisc=addshaperqdisc.start();
  if (successqdisc) {
    statusbar->message(i18n("New qdisc created"), 2000);
    QString msg;
    msg.sprintf( i18n("qdisc created for interface %s"), sfqinterfaceChoice);
    messageLog->insertItem(msg);
    addLog(addshaperqdisc);
    tcdev = 0;
    listinfo.clear();
    subqdiscs.clear();
    startMonitorProcess(); // refresh window
    
  } else {
    statusbar->message(i18n("tc died"), 2000);
    QMessageBox::critical(0, "Error",
       i18n("qdisc has not been created, please check your parameters! \n"));
  }
};


// tbf
void Ktctool::tbfShaper() {

  QBoxLayout *top = new QBoxLayout(tab_6, QBoxLayout::LeftToRight, 10);
  QGridLayout * grid = new QGridLayout(10,11);
  top->addLayout(grid);

  QLabel *labeltbfInterface = new QLabel(tab_6);
  labeltbfInterface->setText(i18n("Interface:"));
  grid->addMultiCellWidget(labeltbfInterface,0,0,0,0,AlignRight);

  tbfinterface = new QComboBox( FALSE, tab_6, "interface");
  tbfinterface->insertStrList(devices);
  tbfinterface->setFixedHeight(tbfinterface->sizeHint().height());
  connect(tbfinterface,SIGNAL(activated(int)), SLOT(tbfinterfaceActivated(int)));
  grid->addMultiCellWidget(tbfinterface,0,0,1,1);

  QLabel *labelrate = new QLabel(tab_6);
  labelrate->setText("rate:");
  grid->addMultiCellWidget(labelrate,0,0,2,2,AlignRight);

  tbfRate = new QLineEdit(tab_6);
  tbfRate->setFixedHeight(tbfRate->sizeHint().height());
  tbfRate->setText("200KB");
  grid->addMultiCellWidget(tbfRate,0,0,3,3);

  QLabel *labellimit = new QLabel(tab_6);
  labellimit->setText("limit:");
  grid->addMultiCellWidget(labellimit,0,0,4,4,AlignRight);

  tbfLimit = new QLineEdit(tab_6);
  tbfLimit->setFixedHeight(tbfLimit->sizeHint().height());
  tbfLimit->setText("500KB");
  grid->addMultiCellWidget(tbfLimit,0,0,5,5);

  QLabel *labelburst = new QLabel(tab_6);
  labelburst->setText("burst:");
  grid->addMultiCellWidget(labelburst,0,0,6,6,AlignRight);

  tbfBurst = new QLineEdit(tab_6);
  tbfBurst->setFixedHeight(tbfBurst->sizeHint().height());
  tbfBurst->setText("20KB");
  grid->addMultiCellWidget(tbfBurst,0,0,7,7);

  QLabel *labelend = new QLabel(tab_6);
  labelend->setText("    ");
  grid->addMultiCellWidget(labelend,0,0,8,8,AlignRight);

  QLabel *labelpeakrate = new QLabel(tab_6);
  labelpeakrate->setText("peakrate:");
  grid->addMultiCellWidget(labelpeakrate,1,1,2,2,AlignRight);

  tbfPeakrate = new QLineEdit(tab_6);
  tbfPeakrate->setFixedHeight(tbfPeakrate->sizeHint().height());
  tbfPeakrate->setText("");
  grid->addMultiCellWidget(tbfPeakrate,1,1,3,3);

  QLabel *labellatency = new QLabel(tab_6);
  labellatency->setText("latency:");
  grid->addMultiCellWidget(labellatency,1,1,4,4,AlignRight);

  tbfLatency = new QLineEdit(tab_6);
  tbfLatency->setFixedHeight(tbfLatency->sizeHint().height());
  tbfLatency->setText("");
  grid->addMultiCellWidget(tbfLatency,1,1,5,5);

  QLabel *labelmtu = new QLabel(tab_6);
  labelmtu->setText("mtu:");
  grid->addMultiCellWidget(labelmtu,1,1,6,6,AlignRight);

  tbfMtu = new QLineEdit(tab_6);
  tbfMtu->setFixedHeight(tbfMtu->sizeHint().height());
  tbfMtu->setText("");
  grid->addMultiCellWidget(tbfMtu,1,1,7,7);

  QLabel *labelqdischandle = new QLabel(tab_6);
  labelqdischandle->setText(i18n("qdisc handle:"));
  grid->addMultiCellWidget(labelqdischandle,1,1,0,0,AlignRight);

  tbfqdiscHandle = new QLineEdit(tab_6);
  tbfqdiscHandle->setFixedHeight(tbfqdiscHandle->sizeHint().height());
  tbfqdiscHandle->setText("1:");
  grid->addMultiCellWidget(tbfqdiscHandle,1,1,1,1);

  QLabel *labelqdiscparent = new QLabel(tab_6);
  labelqdiscparent->setText(i18n("qdisc parent:"));
  grid->addMultiCellWidget(labelqdiscparent,2,2,0,0,AlignRight);

  tbfqdiscParent = new QLineEdit(tab_6);
  tbfqdiscParent->setFixedHeight(tbfqdiscParent->sizeHint().height());
  tbfqdiscParent->setText("");
  grid->addMultiCellWidget(tbfqdiscParent,2,2,1,1);

  QPushButton *btnShaper = new QPushButton(tab_6);
  btnShaper->setText( i18n("Add qdisc"));
  btnShaper->setFixedHeight(btnShaper->sizeHint().height());
  connect(btnShaper,SIGNAL(clicked()), this, SLOT(starttbfQdisc()));
  grid->addWidget(btnShaper,2,2);

  QPushButton *btnChange = new QPushButton(tab_6);
  btnChange->setText( i18n("Change qdisc"));
  btnChange->setFixedHeight(btnChange->sizeHint().height());
  connect(btnChange,SIGNAL(clicked()), this, SLOT(starttbfQdiscChange()));
  grid->addWidget(btnChange,2,3);

  QWhatsThis::add (tbfinterface, i18n("Select network interface."));
  QWhatsThis::add (tbfqdiscHandle, i18n("This is the handle (major:minor) you want to give it. The minor number must be zero."));
  QWhatsThis::add (tbfqdiscParent, i18n("Not required. Parent CLASS ID, e. g. 1:10 (which is specifying the parent of this qdisc). Each qdisc can only be the parent of one other class."));
  QWhatsThis::add (btnShaper, i18n("Add qdisc. Root if qdisc parent field is empty. The TBF implementation consists of a buffer (bucket), constantly filled by some virtual pieces of information called\
 tokens, at a specific rate (token rate). The most important parameter of the bucket is its size, that is the number of tokens it can store."));
  QWhatsThis::add (btnChange, i18n("Change qdisc parameters. The qdisc must exist and must be the same type."));
  QWhatsThis::add (tbfRate, i18n("E.g. 1mbit."));
  QWhatsThis::add (tbfLimit, i18n("Limit is the number of bytes that can be queued waiting for tokens to become available. Clear the latency parameter."));
  QWhatsThis::add (tbfBurst, i18n("Size of the bucket, in bytes. This is the maximum amount of bytes that tokens can be available for instantaneously. In general, larger shaping rates require a larger buffer. For 10mbit/s, you need at least 10KB buffer"));
  QWhatsThis::add (tbfPeakrate, i18n("Not required. If tokens are available, and packets arrive, they are sent out immediately by default. The peakrate can be used to specify how quickly the bucket is allowed to be depleted. This is achieved by releasing\
 a packet, and then wait just long enough, and release the next. However, due to de default 10ms timer resolution of Unix, with 10.000 bits average packets, we are limited to 1mbit/s of peakrate!"));
  QWhatsThis::add (tbfLatency, i18n("Not required. Specifies the maximum amount of time a packet can sit in the TBF, e.g. 50ms. Clear the limit parameter."));
  QWhatsThis::add (tbfMtu, i18n("Not required. Size of the bucket, in bytes. The 1mbit/s peakrate is not very useful if your regular rate is more than that. A higher peakrate is possible by sending out more packets per timertick, which effectively means that\
 we create a second bucket. To calculate the maximum possible peakrate, multiply the configured mtu by 100 (HZ, which is 100 on Intel, 1024 on Alpha)"));

  tab_6->show();
}


// interface choice
void Ktctool::tbfinterfaceActivated(int index) {

  switch(index) {
  default:
    tbfinterfaceChoice = devices.at(0);
  case 0:
    tbfinterfaceChoice = devices.at(0);
    break;
  case 1:
    tbfinterfaceChoice = devices.at(1);
    break;
  case 2:
    tbfinterfaceChoice = devices.at(2);
    break;
  case 3:
    tbfinterfaceChoice = devices.at(3);
    break;
  case 4:
    tbfinterfaceChoice = devices.at(4);
    break;
  case 5:
    tbfinterfaceChoice = devices.at(5);
    break;
  case 6:
    tbfinterfaceChoice = devices.at(6);
    break;
  case 7:
    tbfinterfaceChoice = devices.at(7);
    break;
  case 8:
    tbfinterfaceChoice = devices.at(8);
    break;
  };

};


// if tc object exist
void Ktctool::starttbfQdisc() {
  QString shaperinterface, tcid;

  startfunction = "tbfQdisc";
  shaperinterface.sprintf(tbfinterfaceChoice);
  tcid.sprintf((const char *)(tbfqdiscHandle->text()));
  QString tctype = "qdisc";
  startexistShaper(shaperinterface, tcid, tctype);

};


// start changing parameters
void Ktctool::starttbfQdiscChange() {

  startfunction = "change";
  tbfQdisc();

};


// create qdisc tbf
void Ktctool::tbfQdisc() {
   QString tbfqdischandle, tbfqdiscparent, tbfrate, tbflimit, tbfburst, tbfpeakrate, tbflatency, tbfmtu, start;
   QRegExp isinteger = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itIT]*$");
   QRegExp isbps = QRegExp("^[0-9][.0-9]*[mMKkbB]*[itpsITPS]*$");
   bool successqdisc = false;

   tbfqdischandle.sprintf((const char *)(tbfqdiscHandle->text()));
   tbfqdiscparent.sprintf((const char *)(tbfqdiscParent->text()));
   tbfrate.sprintf((const char *)(tbfRate->text()));
   tbflimit.sprintf((const char *)(tbfLimit->text()));
   tbfburst.sprintf((const char *)(tbfBurst->text()));
   tbfpeakrate.sprintf((const char *)(tbfPeakrate->text()));
   tbflatency.sprintf((const char *)(tbfLatency->text()));
   tbfmtu.sprintf((const char *)(tbfMtu->text()));

   addshaperqdisc.clearArguments();

  if ((tbfqdischandle.length()==0)||(tbfqdischandle.contains(" ")))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc handle! \n e.g. 1! \n"));
    }
  else if (tbfrate.contains(isbps) <= 0)
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc rate! \n"));
    }
  else if ((tbflimit.contains(isinteger) <= 0)&&(tbflatency.length()==0))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc limit or latency! \n"));
    }
  else if ((tbflimit.length()!=0)&&(tbflatency.length()!=0))
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply only the qdisc limit or latency! \n"));
    }
  else if (tbfburst.contains(isinteger) <= 0)
    {
      QMessageBox::critical(0, "Error",
           i18n("You have to supply the qdisc burst! \n"));
    }
  else
  {
      start = "yes";
      if (tcexistid == "no") start = "no";
      addshaperqdisc << "tc" << "qdisc";
      if (startfunction == "change") addshaperqdisc << "change";
      else addshaperqdisc << "add";
      addshaperqdisc << "dev" << tbfinterfaceChoice;
      if (tbfqdiscparent.length()==0) addshaperqdisc << "root";
      else addshaperqdisc << "parent" << tbfqdiscparent;
      addshaperqdisc << "handle" << tbfqdischandle << "tbf" << "rate" << tbfrate;
      if (tbflimit.length()!=0)
        if (tbflimit.contains(isinteger)) addshaperqdisc << "limit" << tbflimit;
        else {
          QMessageBox::critical(0, "Error", i18n("Limit is not an integer! \n"));
          start = "no";
        }
      if (tbfburst.length()!=0)
        if (tbfburst.contains(isinteger)) addshaperqdisc << "burst" << tbfburst;
        else {
          QMessageBox::critical(0, "Error", i18n("Burst is not an integer! \n"));
          start = "no";
        }
      if (tbflatency.length()!=0) addshaperqdisc << "latency" << tbflatency;
      if (tbfpeakrate.contains(isbps)) addshaperqdisc << "peakrate" << tbfpeakrate;
      if (tbfmtu.length()!=0)
        if (tbfmtu.contains(isinteger)) addshaperqdisc << "mtu" << tbfmtu;
        else {
          QMessageBox::critical(0, "Error", i18n("Mtu is not an integer! \n"));
          start = "no";
        }
  }

  if (start == "yes")
    successqdisc=addshaperqdisc.start();
  if (successqdisc) {
    QString msg;
    if (startfunction == "change") {
    statusbar->message(i18n("Qdisc changed"), 2000);
		msg.sprintf( i18n("qdisc changed for interface %s"), tbfinterfaceChoice);
    } else {
    statusbar->message(i18n("New qdisc created"), 2000);
    msg.sprintf( i18n("qdisc created for interface %s"), tbfinterfaceChoice);
    }
    messageLog->insertItem(msg);
    addLog(addshaperqdisc);
    tcdev = 0;
    listinfo.clear();
    subqdiscs.clear();
    startMonitorProcess(); // refresh window
  } else {
    statusbar->message(i18n("tc died"), 2000);
    QMessageBox::critical(0, "Error",
       i18n("qdisc has not been created, please check your parameters! \n"));
  }
};
